/*
 * arm/jit.h
 * Common ARM JIT configuration information.
 *
 * Copyright (c) 1996, 1997
 *	Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file. 
 *
 * Written by Raffaele Sena (raff@aromatic.com) based on m68k/jit.h
 */

#ifndef __arm_jit_h
#define __arm_jit_h

/**/
/* Exception handling information. */
/**/

/* Structure of exception frame on stack */
typedef struct _exceptionFrame {
        uintp	retbp;
        uintp	retpc;
} exceptionFrame;

/* Is this frame valid (ie. is it on the current stack) ? */
#define	FRAMEOKAY(f)							\
	((f)->retbp >= (int)TCTX(currentThread)->stackBase &&		\
	 (f)->retbp < (int)TCTX(currentThread)->stackEnd)

/* Get the next frame in the chain */
#define	NEXTFRAME(f)							\
	(f) = ((exceptionFrame*)(f)->retbp)

/* Extract the PC from the given frame */
#define	PCFRAME(f)							\
	((f)->retpc)

/* Extract a local argument from given frame */
#define	FRAMEOBJECT(f)							\
	(*(Hjava_lang_Object**)(f->retbp + 8))

/* Get the first exception frame from a subroutine call */
#define	FIRSTFRAME(f, o)						\
	(f) = *((exceptionFrame*)(((uintp)&(o))-8))

/* Call the relevant exception handler (rewinding the stack as
   necessary). */
#define CALL_KAFFE_EXCEPTION(frame, info, obj)				\
        asm("   ldr a1, %2						\n\
		ldr fp, %0						\n\
		mov pc, %1						\n\
		" : : "g" (frame->retbp), "r" (info.handler), "g" (obj) : "a1")


/**/
/* Method dispatch.  */
/**/

#define HAVE_TRAMPOLINE

typedef struct _methodTrampoline {
	unsigned int code[2];
	int fixup;
	struct _methods* meth;
} methodTrampoline;

extern void arm_do_fixup_trampoline(void);

#define FILL_IN_TRAMPOLINE(t,m)						\
	do {								\
		(t)->code[0] = 0xe59fc000;	/* ldr ip, fixup */	\
		(t)->code[1] = 0xe1a0f00c;	/* mov pc, ip    */	\
		(t)->fixup = (int)arm_do_fixup_trampoline;		\
		(t)->meth = (m);					\
	} while (0)

#define FIXUP_TRAMPOLINE_DECL	Method* _pmeth
#define FIXUP_TRAMPOLINE_INIT	(meth = _pmeth)


/**/
/* Register management information. */
/**/

/* Define the register set */
#define	REGISTER_SET							\
	{ /* r0/a1 */	0, 0, Rint|Rref,	0, 0, 0    },		\
	{ /* r1/a2 */	0, 0, Rint|Rref,	0, 0, 1    },		\
	{ /* r2/a3 */	0, 0, Rint|Rref,	0, 0, 2    },		\
	{ /* r3/a4 */	0, 0, Rint|Rref,	0, 0, 3    },		\
	{ /* r4/v1 */	0, 0, Rint|Rref,	0, 0, 4    },		\
	{ /* r5/v2 */	0, 0, Rint|Rref,	0, 0, 5    },		\
	{ /* r6/v3 */	0, 0, Rint|Rref,	0, 0, 6    },		\
	{ /* r7/v4 */	0, 0, Rint|Rref,	0, 0, 7    },		\
	{ /* r8/v5 */	0, 0, Rint|Rref,	0, 0, 8    },		\
	{ /* r9/v6 */	0, 0, Rint|Rref,	0, 0, 9    },		\
	{ /* r10/v7 */	0, 0, Rint|Rref,	0, 0, 10   },		\
	{ /* r11/fp */	0, 0, Reserved,		0, 0, 11   },		\
	{ /* r12/ip */	0, 0, Rint|Rref,	0, 0, 12   },		\
	{ /* r13/sp */	0, 0, Reserved,		0, 0, 13   },		\
	{ /* r14/lr */	0, 0, Rint|Rref,	0, 0, 14   },		\
	{ /* r15/pc */	0, 0, Reserved,		0, 0, 15   },		\
	{ /* f0  */	0, 0, Rfloat|Rdouble,	0, 0, 16   },		\
	{ /* f1  */	0, 0, Rfloat|Rdouble,	0, 0, 17   },		\
	{ /* f2  */	0, 0, Rfloat|Rdouble,	0, 0, 18   },		\
	{ /* f3  */	0, 0, Rfloat|Rdouble,	0, 0, 19   },		\
	{ /* f4  */	0, 0, Rfloat|Rdouble,	0, 0, 20   },		\
	{ /* f5  */	0, 0, Rfloat|Rdouble,	0, 0, 21   },		\
	{ /* f6  */	0, 0, Rfloat|Rdouble,	0, 0, 22   },		\
	{ /* f7  */	0, 0, Rfloat|Rdouble,	0, 0, 23   },

/* Number of registers in the register set */
#define	NR_REGISTERS	24

/**/
/* Opcode generation. */
/**/

#define Llong24		(Larchdepend+0)	/* Label is 24 bits long */
#define Llong8x4	(Larchdepend+1)	/* Label is 8 bits long */

#define LABEL_Lframe(P,D,L)						\
	{								\
		int framesize = SLOTSIZE * (maxLocal + maxStack +	\
			maxTemp - maxArgs);				\
		*(uint32*)(P) = framesize;				\
	}

#define	LABEL_Llong24(P,D,L)						\
	{								\
		assert(((D) & 0xFC000003) == 0 				\
			|| ((D) & 0xFE000003) == 0xFE000000);		\
		*(uint32*)(P) |= ((((D)-8) >> 2) & 0x00FFFFFF);		\
	}

#define	LABEL_Llong8x4(P,D,L)						\
	{								\
		((uint32*)(P))[0] |= ((D) & 0x000000FF) >>  0;		\
		((uint32*)(P))[1] |= ((D) & 0x0000FF00) >>  8;		\
		((uint32*)(P))[2] |= ((D) & 0x00FF0000) >> 16;		\
		((uint32*)(P))[3] |= ((D) & 0xFF000000) >> 24;		\
	}

#define	EXTRA_LABELS(P,D,L)						\
	case Lframe:	LABEL_Lframe(P,D,L);	break;			\
	case Llong24:	LABEL_Llong24(P,D,L);	break;			\
	case Llong8x4:	LABEL_Llong8x4(P,D,L);	break;

/* Define if generated code uses two operands rather than three */
#undef	TWO_OPERAND	/* ARM processor supports 3 operands */

/**/
/* Slot management information. */
/**/

/* Size of each slot */
#define	SLOTSIZE	4

/* Generate slot offset for an argument */
#define SLOT2ARGOFFSET(_n)	(8 + SLOTSIZE * (_n))

/* Generate slot offset for a local (non-argument) */
#define SLOT2LOCALOFFSET(_n)	(-SLOTSIZE * (maxTemp+maxLocal+maxStack - (_n)))

/* Wrap up a native call for the JIT */
#define KAFFEJIT_TO_NATIVE(_m)

/* We DO have to flush the code out of cache on the arm */
#if 0
#define FLUSH_DCACHE(beg,end)	__clear_cache((beg), (end))
#else
#define FLUSH_DCACHE(beg,end)	arm_clear_cache((beg), (end))
#endif

#endif
